@@ -1,7 +1,7 @@
 // synthv1_config.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,8 +27,8 @@
 #define SYNTHV1_TITLE       PACKAGE_NAME
 
 #define SYNTHV1_SUBTITLE    "an old-school polyphonic synthesizer."
-#define SYNTHV1_WEBSITE     "http://synthv1.sourceforge.net"
-#define SYNTHV1_COPYRIGHT   "Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved."
+#define SYNTHV1_WEBSITE     "https://synthv1.sourceforge.io"
+#define SYNTHV1_COPYRIGHT   "Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved."
 
 #define SYNTHV1_DOMAIN      "rncbc.org"
 
@@ -63,6 +63,9 @@ public:
 	int iKnobDialMode;
 	int iKnobEditMode;
 
+	// Default randomize factor (percent).
+	float fRandomizePercent;
+
 	// Special persistent options.
 	bool bControlsEnabled;
 	bool bProgramsEnabled;
@@ -71,9 +74,19 @@ public:
 	// Run-time special non-persistent options.
 	bool bDontUseNativeDialogs;
 
-	// Custom widget style theme.
+	// Custom color palette/widget style themes.
+	QString sCustomColorTheme;
 	QString sCustomStyleTheme;
 
+	// Micro-tuning options.
+	bool    bTuningEnabled;
+	float   fTuningRefPitch;
+	int     iTuningRefNote;
+	QString sTuningScaleDir;
+	QString sTuningScaleFile;
+	QString sTuningKeyMapDir;
+	QString sTuningKeyMapFile;
+
 	// Singleton instance accessor.
 	static synthv1_config *getInstance();
 
