@@ -1,7 +1,7 @@
 // synthv1_controls.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,6 +23,8 @@
 
 #include <QHash>
 
+#include <math.h>
+
 
 #define RPN_MSB   0x65
 #define RPN_LSB   0x64
@@ -216,15 +218,21 @@ public:
 			synthv1_controls::Event *old_events = m_events;
 			if (old_events) {
 				if (m_write > m_read) {
-					::memcpy(new_events + m_read, old_events + m_read,
+					::memcpy(
+						(void *) (new_events + m_read),
+						(void *) (old_events + m_read),
 						(m_write - m_read) * sizeof(synthv1_controls::Event));
 				}
 				else
 				if (m_write < m_read) {
-					::memcpy(new_events + m_read, old_events + m_read,
+					::memcpy(
+						(void *) (new_events + m_read),
+						(void *) (old_events + m_read),
 						(old_size - m_read) * sizeof(synthv1_controls::Event));
 					if (m_write > 0) {
-						::memcpy(new_events + old_size, old_events,
+						::memcpy(
+							(void *) (new_events + old_size),
+							(void *) (old_events),
 							m_write * sizeof(synthv1_controls::Event));
 					}
 					m_write += old_size;
@@ -334,7 +342,7 @@ public:
 			xrpn_item& item = get_item(channel);
 			if (item.is_any() && item.type() != synthv1_controls::RPN)
 				enqueue(item);
-			if (item.is_status() // RPN NULL [MSB]
+			if (item.is_status() // RPN nullptr [MSB]
 				&& item.type() == synthv1_controls::RPN && item.is_param_lsb()
 				&& item.param_lsb() == 0x7f && event.value == 0x7f) {
 				item.clear();
@@ -358,7 +366,7 @@ public:
 			xrpn_item& item = get_item(channel);
 			if (item.is_any() && item.type() != synthv1_controls::RPN)
 				enqueue(item);
-			if (item.is_status() // RPN NULL [LSB]
+			if (item.is_status() // RPN nullptr [LSB]
 				&& item.type() == synthv1_controls::RPN && item.is_param_msb()
 				&& item.param_msb() == 0x7f && event.value == 0x7f) {
 				item.clear();
@@ -450,14 +458,15 @@ public:
 				--m_count;
 			}
 			else
-			if (item.is_param_msb() || item.is_value_msb()
-				|| (item.type() == synthv1_controls::CC14
+			if ((item.is_param_msb() && item.is_value_msb())
+				|| (item.type() == synthv1_controls::CC14 && item.is_param_lsb()
 					&& item.param_lsb() != event.key.param + CC14_LSB_MIN))
 				enqueue(item);
 			if (!item.is_status()) {
 				item.set_status(synthv1_controls::CC14 | channel);
 				++m_count;
 			}
+			item.set_param_lsb(event.key.param + CC14_LSB_MIN);
 			item.set_param_msb(event.key.param);
 			item.set_value_msb(event.value);
 			if (item.is_14bit())
@@ -473,14 +482,15 @@ public:
 				--m_count;
 			}
 			else
-			if (item.is_param_lsb() || item.is_value_lsb()
-				|| (item.type() == synthv1_controls::CC14
+			if ((item.is_param_lsb() && item.is_value_lsb())
+				|| (item.type() == synthv1_controls::CC14 && item.is_param_msb()
 					&& item.param_msb() != event.key.param - CC14_LSB_MIN))
 				enqueue(item);
 			if (!item.is_status()) {
 				item.set_status(synthv1_controls::CC14 | channel);
 				++m_count;
 			}
+			item.set_param_msb(event.key.param - CC14_LSB_MIN);
 			item.set_param_lsb(event.key.param);
 			item.set_value_lsb(event.value);
 			if (item.is_14bit())
@@ -504,16 +514,20 @@ protected:
 		if (item.type() == synthv1_controls::CC14) {
 			if (item.is_14bit()) {
 				m_queue.push(item.status(), item.param_msb(), item.value());
+				const unsigned char value_msb = item.value_msb();
+				item.clear_value();
+				item.set_value_msb(value_msb);
+			//	--m_count;
 			} else  {
 				const unsigned short status
 					= synthv1_controls::CC | item.channel();
-				if (item.is_value_msb())
+				if (item.is_param_msb() && item.is_value_msb())
 					m_queue.push(status, item.param_msb(), item.value_msb());
-				if (item.is_value_lsb())
+				if (item.is_param_lsb() && item.is_value_lsb())
 					m_queue.push(status, item.param_lsb(), item.value_lsb());
+				item.clear();
+				--m_count;
 			}
-			item.clear();
-			--m_count;
 		}
 		else
 		if (item.is_ready()) {
@@ -653,8 +667,8 @@ void synthv1_controls::process_event ( const Event& event )
 		const float v0 = data.val;
 		const float v1 = synthv1_param::paramScale(index,
 			m_sched_in.instance()->paramValue(index));
-		const float d1 = (v1 - fScale);
-		const float d2 = (v1 - v0) * d1;
+		const float d1 = ::fabsf(v1 - fScale);
+		const float d2 = ::fabsf(v1 - v0) * d1;
 		bSync = (d2 < 0.001f);
 		if (bSync) {
 			data.val = fScale;
