@@ -264,7 +264,7 @@ public:
 			// process
 			const float lo = m_lo.output(m_mi.output(m_hi.output(*in + ad)));
 			// compute peak
-			const float peak = ::fabs(lo);
+			const float peak = ::fabsf(lo);
 			// compute gain
 			float gain = 1.0f;
 			if (peak > threshold)
@@ -508,7 +508,7 @@ private:
 
 
 //-------------------------------------------------------------------------
-// synthv1_fx_allpass_delay - All-pass delay implementation.
+// synthv1_fx_allpass - All-pass delay implementation.
 
 class synthv1_fx_allpass
 {
