@@ -1,7 +1,7 @@
 // synthv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -67,9 +67,13 @@ public:
 	void sessionEvent(void *pvSessionArg);
 #endif
 
+	void shutdown();
+	void shutdown_close();
+
 protected:
  
 	void updatePreset(bool bDirty);
+	void updateTuning();
 
 private:
 
@@ -115,6 +119,9 @@ class synthv1widget_jack;
 class synthv1_nsm;
 #endif
 
+#ifdef HAVE_SIGNAL_H
+class QSocketNotifier;
+#endif
 
 class synthv1_jack_application : public QObject
 {
@@ -131,19 +138,33 @@ public:
 	// Facade method.
 	int exec();
 
-#ifdef CONFIG_NSM
+	// JACK shutdown handler.
+	void shutdown();
+
+	// Pseudo-singleton accessor.
+	static synthv1_jack_application *getInstance();
+
+signals:
+
+	void shutdown_signal();
 
 protected slots:
 
+#ifdef CONFIG_NSM
 	// NSM callback slots.
 	void openSession();
 	void saveSession();
-
 	void hideSession();
 	void showSession();
-
 #endif	// CONFIG_NSM
 
+#ifdef HAVE_SIGNAL_H
+	// SIGTERM signal handler.
+	void handle_sigterm();
+#endif
+
+	void shutdown_slot();
+
 protected:
 
 	// Argument parser method.
@@ -157,6 +178,7 @@ private:
 	// Instance variables.
 	QCoreApplication *m_pApp;
 	bool m_bGui;
+
 	QStringList m_presets;
 
 	synthv1_jack *m_pSynth;
@@ -165,6 +187,12 @@ private:
 #ifdef CONFIG_NSM
 	synthv1_nsm *m_pNsmClient;
 #endif
+
+#ifdef HAVE_SIGNAL_H
+	QSocketNotifier *m_pSigtermNotifier;
+#endif
+
+	static synthv1_jack_application *g_pInstance;
 };
 
 
