@@ -1,7 +1,7 @@
 // synthv1_lv2ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -61,17 +61,13 @@ void synthv1_lv2ui::write_function (
 // synthv1_lv2ui - LV2 UI desc.
 //
 
-static QApplication *synthv1_lv2ui_qapp_instance = NULL;
-static unsigned int  synthv1_lv2ui_qapp_refcount = 0;
-
-
 static LV2UI_Handle synthv1_lv2ui_instantiate (
 	const LV2UI_Descriptor *, const char *, const char *,
 	LV2UI_Write_Function write_function,
 	LV2UI_Controller controller, LV2UI_Widget *widget,
 	const LV2_Feature *const *features )
 {
-	synthv1_lv2 *pSynth = NULL;
+	synthv1_lv2 *pSynth = nullptr;
 
 	for (int i = 0; features && features[i]; ++i) {
 		if (::strcmp(features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0) {
@@ -80,15 +76,8 @@ static LV2UI_Handle synthv1_lv2ui_instantiate (
 		}
 	}
 
-	if (pSynth == NULL)
-		return NULL;
-
-	if (qApp == NULL && synthv1_lv2ui_qapp_instance == NULL) {
-		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, NULL };
-		synthv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
-	}
-	synthv1_lv2ui_qapp_refcount++;
+	if (pSynth == nullptr)
+		return nullptr;
 
 	synthv1widget_lv2 *pWidget
 		= new synthv1widget_lv2(pSynth, controller, write_function);
@@ -99,13 +88,8 @@ static LV2UI_Handle synthv1_lv2ui_instantiate (
 static void synthv1_lv2ui_cleanup ( LV2UI_Handle ui )
 {
 	synthv1widget_lv2 *pWidget = static_cast<synthv1widget_lv2 *> (ui);
-	if (pWidget) {
+	if (pWidget)
 		delete pWidget;
-		if (--synthv1_lv2ui_qapp_refcount == 0 && synthv1_lv2ui_qapp_instance) {
-			delete synthv1_lv2ui_qapp_instance;
-			synthv1_lv2ui_qapp_instance = NULL;
-		}
-	}
 }
 
 static void synthv1_lv2ui_port_event (
@@ -173,6 +157,29 @@ static const LV2UI_Show_Interface synthv1_lv2ui_show_interface =
 
 #endif	// CONFIG_LV2_UI_IDLE
 
+
+#ifdef CONFIG_LV2_UI_RESIZE
+
+int synthv1_lv2ui_resize ( LV2UI_Handle ui, int width, int height )
+{
+	synthv1widget_lv2 *pWidget = static_cast<synthv1widget_lv2 *> (ui);
+	if (pWidget) {
+		pWidget->resize(width, height);
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+static const LV2UI_Resize synthv1_lv2ui_resize_interface =
+{
+	nullptr, // handle: host should use its own when calling ui_resize().
+	synthv1_lv2ui_resize
+};
+
+#endif	// CONFIG_LV2_UI_RESIZE
+
+
 static const void *synthv1_lv2ui_extension_data ( const char *uri )
 {
 #ifdef CONFIG_LV2_UI_IDLE
@@ -185,11 +192,60 @@ static const void *synthv1_lv2ui_extension_data ( const char *uri )
 		return (void *) &synthv1_lv2ui_show_interface;
 	else
 #endif
-	return NULL;
+#ifdef CONFIG_LV2_UI_RESIZE
+	if (::strcmp(uri, LV2_UI__resize) == 0)
+		return (void *) &synthv1_lv2ui_resize_interface;
+	else
+#endif
+	return nullptr;
 }
 
 
-#ifdef CONFIG_LV2_EXTERNAL_UI
+#ifdef CONFIG_LV2_UI_X11
+
+static LV2UI_Handle synthv1_lv2ui_x11_instantiate (
+	const LV2UI_Descriptor *, const char *, const char *,
+	LV2UI_Write_Function write_function,
+	LV2UI_Controller controller, LV2UI_Widget *widget,
+	const LV2_Feature *const *ui_features )
+{
+	WId winid, parent = 0;
+	LV2UI_Resize *resize = nullptr;
+	synthv1_lv2 *pSynth = nullptr;
+
+	for (int i = 0; ui_features[i]; ++i) {
+		if (::strcmp(ui_features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0)
+			pSynth = static_cast<synthv1_lv2 *> (ui_features[i]->data);
+		else
+		if (::strcmp(ui_features[i]->URI, LV2_UI__parent) == 0)
+			parent = (WId) ui_features[i]->data;
+		else
+		if (::strcmp(ui_features[i]->URI, LV2_UI__resize) == 0)
+			resize = (LV2UI_Resize *) ui_features[i]->data;
+	}
+
+	if (pSynth == nullptr)
+		return nullptr;
+	if (!parent)
+		return nullptr;
+
+	synthv1widget_lv2 *pWidget
+		= new synthv1widget_lv2(pSynth, controller, write_function);
+	if (resize && resize->handle) {
+		const QSize& hint = pWidget->sizeHint();
+		resize->ui_resize(resize->handle, hint.width(), hint.height());
+	}
+	winid = pWidget->winId();
+	pWidget->windowHandle()->setParent(QWindow::fromWinId(parent));
+	pWidget->show();
+	*widget = (LV2UI_Widget) winid;
+	return pWidget;
+}
+
+#endif	// CONFIG_LV2_UI_X11
+
+
+#ifdef CONFIG_LV2_UI_EXTERNAL
 
 struct synthv1_lv2ui_external_widget
 {
@@ -233,8 +289,8 @@ static LV2UI_Handle synthv1_lv2ui_external_instantiate (
 	LV2UI_Controller controller, LV2UI_Widget *widget,
 	const LV2_Feature *const *ui_features )
 {
-	synthv1_lv2 *pSynth = NULL;
-	LV2_External_UI_Host *external_host = NULL;
+	synthv1_lv2 *pSynth = nullptr;
+	LV2_External_UI_Host *external_host = nullptr;
 
 	for (int i = 0; ui_features[i] && !external_host; ++i) {
 		if (::strcmp(ui_features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0)
@@ -246,13 +302,6 @@ static LV2UI_Handle synthv1_lv2ui_external_instantiate (
 		}
 	}
 
-	if (qApp == NULL && synthv1_lv2ui_qapp_instance == NULL) {
-		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, NULL };
-		synthv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
-	}
-	synthv1_lv2ui_qapp_refcount++;
-
 	synthv1_lv2ui_external_widget *pExtWidget = new synthv1_lv2ui_external_widget;
 	pExtWidget->external.run  = synthv1_lv2ui_external_run;
 	pExtWidget->external.show = synthv1_lv2ui_external_show;
@@ -272,10 +321,6 @@ static void synthv1_lv2ui_external_cleanup ( LV2UI_Handle ui )
 		if (pExtWidget->widget)
 			delete pExtWidget->widget;
 		delete pExtWidget;
-		if (--synthv1_lv2ui_qapp_refcount == 0 && synthv1_lv2ui_qapp_instance) {
-			delete synthv1_lv2ui_qapp_instance;
-			synthv1_lv2ui_qapp_instance = NULL;
-		}
 	}
 }
 
@@ -291,10 +336,10 @@ static void synthv1_lv2ui_external_port_event (
 
 static const void *synthv1_lv2ui_external_extension_data ( const char * )
 {
-	return NULL;
+	return nullptr;
 }
 
-#endif	// CONFIG_LV2_EXTERNAL_UI
+#endif	// CONFIG_LV2_UI_EXTERNAL
 
 
 static const LV2UI_Descriptor synthv1_lv2ui_descriptor =
@@ -306,7 +351,18 @@ static const LV2UI_Descriptor synthv1_lv2ui_descriptor =
 	synthv1_lv2ui_extension_data
 };
 
-#ifdef CONFIG_LV2_EXTERNAL_UI
+#ifdef CONFIG_LV2_UI_X11
+static const LV2UI_Descriptor synthv1_lv2ui_x11_descriptor =
+{
+	SYNTHV1_LV2UI_X11_URI,
+	synthv1_lv2ui_x11_instantiate,
+	synthv1_lv2ui_cleanup,
+	synthv1_lv2ui_port_event,
+	synthv1_lv2ui_extension_data
+};
+#endif	// CONFIG_LV2_UI_X11
+
+#ifdef CONFIG_LV2_UI_EXTERNAL
 static const LV2UI_Descriptor synthv1_lv2ui_external_descriptor =
 {
 	SYNTHV1_LV2UI_EXTERNAL_URI,
@@ -315,7 +371,7 @@ static const LV2UI_Descriptor synthv1_lv2ui_external_descriptor =
 	synthv1_lv2ui_external_port_event,
 	synthv1_lv2ui_external_extension_data
 };
-#endif	// CONFIG_LV2_EXTERNAL_UI
+#endif	// CONFIG_LV2_UI_EXTERNAL
 
 
 LV2_SYMBOL_EXPORT const LV2UI_Descriptor *lv2ui_descriptor ( uint32_t index )
@@ -323,12 +379,17 @@ LV2_SYMBOL_EXPORT const LV2UI_Descriptor *lv2ui_descriptor ( uint32_t index )
 	if (index == 0)
 		return &synthv1_lv2ui_descriptor;
 	else
-#ifdef CONFIG_LV2_EXTERNAL_UI
+#ifdef CONFIG_LV2_UI_X11
 	if (index == 1)
+		return &synthv1_lv2ui_x11_descriptor;
+	else
+#endif
+#ifdef CONFIG_LV2_UI_EXTERNAL
+	if (index == 2)
 		return &synthv1_lv2ui_external_descriptor;
 	else
 #endif
-	return NULL;
+	return nullptr;
 }
 
 
