@@ -1,7 +1,7 @@
 // synthv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -64,6 +64,7 @@ struct ParamInfo {
 	{ "DCO1_TUNING",   PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // DCO1 Tuning
 	{ "DCO1_GLIDE",    PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCO1 Glide
 	{ "DCO1_ENVTIME",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCO1 Env.Time
+	{ "DCF1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCF1 Enabled
 	{ "DCF1_CUTOFF",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Cutoff
 	{ "DCF1_RESO",     PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCF1 Resonance
 	{ "DCF1_TYPE",     PARAM_INT,     0.0f,   0.0f,   3.0f }, // DCF1 Type
@@ -73,6 +74,7 @@ struct ParamInfo {
 	{ "DCF1_DECAY",    PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DCF1 Decay
 	{ "DCF1_SUSTAIN",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Sustain
 	{ "DCF1_RELEASE",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Release
+	{ "LFO1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // LFO1 Enabled
 	{ "LFO1_SHAPE",    PARAM_INT,     1.0f,   0.0f,   4.0f }, // LFO1 Wave Shape
 	{ "LFO1_WIDTH",    PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO1 Wave Width
 	{ "LFO1_BPM",      PARAM_FLOAT, 180.0f,   0.0f, 360.0f }, // LFO1 BPM
@@ -123,6 +125,7 @@ struct ParamInfo {
 	{ "DCO2_TUNING",   PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // DCO2 Tuning
 	{ "DCO2_GLIDE",    PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCO2 Glide
 	{ "DCO2_ENVTIME",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCO2 Env.Time
+	{ "DCF2_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCF2 Enabled
 	{ "DCF2_CUTOFF",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF2 Cutoff
 	{ "DCF2_RESO",     PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCF2 Resonance
 	{ "DCF2_TYPE",     PARAM_INT,     0.0f,   0.0f,   3.0f }, // DCF2 Type
@@ -132,6 +135,7 @@ struct ParamInfo {
 	{ "DCF2_DECAY",    PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DCF2 Decay
 	{ "DCF2_SUSTAIN",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF2 Sustain
 	{ "DCF2_RELEASE",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF2 Release
+	{ "LFO2_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // LFO2 Enabled
 	{ "LFO2_SHAPE",    PARAM_INT,     1.0f,   0.0f,   4.0f }, // LFO2 Wave Shape
 	{ "LFO2_WIDTH",    PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO2 Wave Width
 	{ "LFO2_BPM",      PARAM_FLOAT, 180.0f,   0.0f, 360.0f }, // LFO2 BPM
@@ -190,7 +194,10 @@ struct ParamInfo {
 	{ "REV1_FEEDB",    PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // Reverb Feedback
 	{ "REV1_WIDTH",    PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // Reverb Width
 	{ "DYN1_COMPRESS", PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // Dynamic Compressor
-	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }  // Dynamic Limiter
+	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // Dynamic Limiter
+
+	{ "KEY1_LOW",      PARAM_INT,     0.0f,   0.0f, 127.0f }, // Keyboard Low
+	{ "KEY1_HIGH",     PARAM_INT,   127.0f,   0.0f, 127.0f }  // Keyboard High
 };
 
 
@@ -206,6 +213,25 @@ float synthv1_param::paramDefaultValue ( synthv1::ParamIndex index )
 }
 
 
+float synthv1_param::paramSafeValue ( synthv1::ParamIndex index, float fValue )
+{
+	const ParamInfo& param = synthv1_params[index];
+
+	if (param.type == PARAM_BOOL)
+		return (fValue > 0.5f ? 1.0f : 0.0f);
+
+	if (fValue < param.min)
+		return param.min;
+	if (fValue > param.max)
+		return param.max;
+
+	if (param.type == PARAM_INT)
+		return ::rintf(fValue);
+	else
+		return fValue;
+}
+
+
 float synthv1_param::paramValue ( synthv1::ParamIndex index, float fScale )
 {
 	const ParamInfo& param = synthv1_params[index];
@@ -244,12 +270,11 @@ bool synthv1_param::paramFloat ( synthv1::ParamIndex index )
 }
 
 
-
 // Preset serialization methods.
 bool synthv1_param::loadPreset (
 	synthv1 *pSynth, const QString& sFilename )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return false;
 
 	QFileInfo fi(sFilename);
@@ -270,6 +295,11 @@ bool synthv1_param::loadPreset (
 	if (!file.open(QIODevice::ReadOnly))
 		return false;
 
+	const bool running = pSynth->running(false);
+
+	pSynth->setTuningEnabled(false);
+	pSynth->reset();
+
 	static QHash<QString, synthv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
 		for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
@@ -309,17 +339,24 @@ bool synthv1_param::loadPreset (
 								index = s_hash.value(sName);
 							}
 							const float fValue = eParam.text().toFloat();
-							pSynth->setParamValue(index, fValue);
+							pSynth->setParamValue(index,
+								synthv1_param::paramSafeValue(index, fValue));
 						}
 					}
 				}
+				else
+				if (eChild.tagName() == "tuning") {
+					synthv1_param::loadTuning(pSynth, eChild);
+				}
 			}
 		}
 	}
 
 	file.close();
 
+	pSynth->stabilize();
 	pSynth->reset();
+	pSynth->running(running);
 
 	QDir::setCurrent(currentDir.absolutePath());
 
@@ -328,11 +365,13 @@ bool synthv1_param::loadPreset (
 
 
 bool synthv1_param::savePreset (
-	synthv1 *pSynth, const QString& sFilename, bool /*bSymLink*/ )
+	synthv1 *pSynth, const QString& sFilename, bool bSymLink )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return false;
 
+	pSynth->stabilize();
+
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
@@ -353,6 +392,13 @@ bool synthv1_param::savePreset (
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
+
+	if (pSynth->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		synthv1_param::saveTuning(pSynth, doc, eTuning, bSymLink);
+		ePreset.appendChild(eTuning);
+	}
+
 	doc.appendChild(ePreset);
 
 	QFile file(fi.filePath());
@@ -368,4 +414,126 @@ bool synthv1_param::savePreset (
 }
 
 
+// Tuning serialization methods.
+void synthv1_param::loadTuning (
+	synthv1 *pSynth, const QDomElement& eTuning )
+{
+	if (pSynth == nullptr)
+		return;
+
+	pSynth->setTuningEnabled(eTuning.attribute("enabled").toInt() > 0);
+
+	for (QDomNode nChild = eTuning.firstChild();
+			!nChild.isNull();
+				nChild = nChild.nextSibling()) {
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+		if (eChild.tagName() == "enabled") {
+			pSynth->setTuningEnabled(eChild.text().toInt() > 0);
+		}
+		if (eChild.tagName() == "ref-pitch") {
+			pSynth->setTuningRefPitch(eChild.text().toFloat());
+		}
+		else
+		if (eChild.tagName() == "ref-note") {
+			pSynth->setTuningRefNote(eChild.text().toInt());
+		}
+		else
+		if (eChild.tagName() == "scale-file") {
+			const QString& sScaleFile
+				= eChild.text();
+			const QByteArray aScaleFile
+				= synthv1_param::loadFilename(sScaleFile).toUtf8();
+			pSynth->setTuningScaleFile(aScaleFile.constData());
+		}
+		else
+		if (eChild.tagName() == "keymap-file") {
+			const QString& sKeyMapFile
+				= eChild.text();
+			const QByteArray aKeyMapFile
+				= synthv1_param::loadFilename(sKeyMapFile).toUtf8();
+			pSynth->setTuningScaleFile(aKeyMapFile.constData());
+		}
+	}
+
+	// Consolidate tuning state...
+	pSynth->resetTuning();
+}
+
+
+void synthv1_param::saveTuning (
+	synthv1 *pSynth, QDomDocument& doc, QDomElement& eTuning, bool bSymLink )
+{
+	if (pSynth == nullptr)
+		return;
+
+	eTuning.setAttribute("enabled", int(pSynth->isTuningEnabled()));
+
+	QDomElement eRefPitch = doc.createElement("ref-pitch");
+	eRefPitch.appendChild(doc.createTextNode(
+		QString::number(pSynth->tuningRefPitch())));
+	eTuning.appendChild(eRefPitch);
+
+	QDomElement eRefNote = doc.createElement("ref-note");
+	eRefNote.appendChild(doc.createTextNode(
+		QString::number(pSynth->tuningRefNote())));
+	eTuning.appendChild(eRefNote);
+
+	const char *pszScaleFile = pSynth->tuningScaleFile();
+	if (pszScaleFile) {
+		const QString& sScaleFile
+			= QString::fromUtf8(pszScaleFile);
+		if (!sScaleFile.isEmpty()) {
+			QDomElement eScaleFile = doc.createElement("scale-file");
+			eScaleFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					synthv1_param::saveFilename(sScaleFile, bSymLink))));
+			eTuning.appendChild(eScaleFile);
+		}
+	}
+
+	const char *pszKeyMapFile = pSynth->tuningKeyMapFile();
+	if (pszKeyMapFile) {
+		const QString& sKeyMapFile
+			= QString::fromUtf8(pszKeyMapFile);
+		if (!sKeyMapFile.isEmpty()) {
+			QDomElement eKeyMapFile = doc.createElement("keymap-file");
+			eKeyMapFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					synthv1_param::saveFilename(sKeyMapFile, bSymLink))));
+			eTuning.appendChild(eKeyMapFile);
+		}
+	}
+}
+
+
+// Load/save and convert canonical/absolute filename helpers.
+QString synthv1_param::loadFilename ( const QString& sFilename )
+{
+	QFileInfo fi(sFilename);
+	if (fi.isSymLink())
+		fi.setFile(fi.symLinkTarget());
+	return fi.canonicalFilePath();
+}
+
+
+QString synthv1_param::saveFilename ( const QString& sFilename, bool bSymLink )
+{
+	QFileInfo fi(sFilename);
+	if (bSymLink && fi.absolutePath() != QDir::current().absolutePath()) {
+		const QString& sPath = fi.absoluteFilePath();
+		const QString& sName = fi.baseName();
+		const QString& sExt  = fi.completeSuffix();
+		const QString& sLink = sName
+			+ '-' + QString::number(qHash(sPath), 16)
+			+ '.' + sExt;
+		QFile(sPath).link(sLink);
+		fi.setFile(QDir::current(), sLink);
+	}
+	else if (fi.isSymLink()) fi.setFile(fi.symLinkTarget());
+	return fi.absoluteFilePath();
+}
+
+
 // end of synthv1_param.cpp
