@@ -1,7 +1,7 @@
 // synthv1_ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -110,4 +110,88 @@ void synthv1_ui::directNoteOn ( int note, int vel )
 }
 
 
+void synthv1_ui::setTuningEnabled ( bool enabled )
+{
+	m_pSynth->setTuningEnabled(enabled);
+}
+
+bool synthv1_ui::isTuningEnabled (void) const
+{
+	return m_pSynth->isTuningEnabled();
+}
+
+
+void synthv1_ui::setTuningRefPitch ( float refPitch )
+{
+	m_pSynth->setTuningRefPitch(refPitch);
+}
+
+float synthv1_ui::tuningRefPitch (void) const
+{
+	return m_pSynth->tuningRefPitch();
+}
+
+
+void synthv1_ui::setTuningRefNote ( int refNote )
+{
+	m_pSynth->setTuningRefNote(refNote);
+}
+
+int synthv1_ui::tuningRefNote (void) const
+{
+	return m_pSynth->tuningRefNote();
+}
+
+
+void synthv1_ui::setTuningScaleFile ( const char *pszScaleFile )
+{
+	m_pSynth->setTuningScaleFile(pszScaleFile);
+}
+
+const char *synthv1_ui::tuningScaleFile (void) const
+{
+	return m_pSynth->tuningScaleFile();
+}
+
+
+void synthv1_ui::setTuningKeyMapFile ( const char *pszKeyMapFile )
+{
+	m_pSynth->setTuningKeyMapFile(pszKeyMapFile);
+}
+
+const char *synthv1_ui::tuningKeyMapFile (void) const
+{
+	return m_pSynth->tuningKeyMapFile();
+}
+
+
+void synthv1_ui::resetTuning (void)
+{
+	m_pSynth->resetTuning();
+}
+
+
+// MIDI note/octave name helper (static).
+QString synthv1_ui::noteName ( int note )
+{
+	static const char *s_notes[] =
+	{
+		QT_TR_NOOP("C"),
+		QT_TR_NOOP("C#/Db"),
+		QT_TR_NOOP("D"),
+		QT_TR_NOOP("D#/Eb"),
+		QT_TR_NOOP("E"),
+		QT_TR_NOOP("F"),
+		QT_TR_NOOP("F#/Gb"),
+		QT_TR_NOOP("G"),
+		QT_TR_NOOP("G#/Ab"),
+		QT_TR_NOOP("A"),
+		QT_TR_NOOP("A#/Bb"),
+		QT_TR_NOOP("B")
+	};
+
+	return QString("%1 %2").arg(s_notes[note % 12]).arg((note / 12) - 1);
+}
+
+
 // end of synthv1_ui.cpp
