@@ -1,7 +1,7 @@
 // synthv1_wave.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,7 +38,7 @@ public:
 
 	// ctor.
 	synthv1_wave_sched (synthv1_wave *wave)
-		: synthv1_sched(NULL, Wave), m_wave(wave) {}
+		: synthv1_sched(nullptr, Wave), m_wave(wave) {}
 
 	// process reset (virtual).
 	void process(int)
@@ -59,8 +59,8 @@ private:
 synthv1_wave::synthv1_wave ( uint32_t nsize, uint16_t nover, uint16_t ntabs )
 	: m_nsize(nsize), m_nover(nover), m_ntabs(ntabs),
 		m_shape(Saw), m_width(1.0f), m_bandl(false),
-		m_srate(44100.0f), m_srand(0),
-		m_min_freq(0.0f), m_max_freq(0.0f), m_sched(NULL)
+		m_srate(44100.0f), m_phase0(0.0f), m_srand(0),
+		m_min_freq(0.0f), m_max_freq(0.0f), m_sched(nullptr)
 {
 	const uint16_t ntabs1 = m_ntabs + 1;
 
@@ -121,7 +121,7 @@ void synthv1_wave::reset_sync (void)
 		break;
 	case Noise:
 		reset_noise();
-		// thru...
+		// Fall thru...
 	default:
 		break;
 	}
@@ -438,19 +438,25 @@ void synthv1_wave::reset_normalize ( uint16_t itab )
 	uint32_t i;
 
 	float pmax = 0.0f;
-	float pmin = 0.0f;
+	float pmid = 0.0f;
 
-	for (i = 0; i < m_nsize; ++i) {
-		const float p = frames[i];
-		if (pmax < p)
-			pmax = p;
-		else
-		if (pmin > p)
-			pmin = p;
+	if (m_ntabs > 0) {
+		for (i = 0; i < m_nsize; ++i)
+			pmid += frames[i];
+		pmid /= float(m_nsize);
+	} else {
+		for (i = 0; i < m_nsize; ++i) {
+			const float p = frames[i];
+			if (pmax < p)
+				pmax = p;
+			else
+			if (pmid > p)
+				pmid = p;
+		}
+		pmid = 0.5f * (pmax + pmid);
+		pmax = 0.0f;
 	}
 
-	const float pmid = 0.5f * (pmax + pmin);
-
 	pmax = 0.0f;
 	for (i = 0; i < m_nsize; ++i) {
 		frames[i] -= pmid;
@@ -477,14 +483,14 @@ void synthv1_wave::reset_interp ( uint16_t itab )
 		frames[i] = frames[i - m_nsize];
 
 	if (itab == m_ntabs) {
-		uint32_t pk = 0;
+		uint32_t k = 0;
 		for (i = 1; i < m_nsize; ++i) {
 			const float p1 = frames[i - 1];
 			const float p2 = frames[i];
 			if (p1 < 0.0f && p2 >= 0.0f)
-				pk = i;
+				k = i;
 		}
-		m_phase0 = float(pk);
+		m_phase0 = float(k) / float(m_nsize);
 	}
 }
 
