@@ -1,7 +1,7 @@
 // synthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include "synthv1widget_config.h"
 #include "synthv1widget_control.h"
 
+#include "synthv1widget_keybd.h"
+
 #include "synthv1_controls.h"
 #include "synthv1_programs.h"
 
@@ -40,6 +42,10 @@
 #include <QShowEvent>
 #include <QHideEvent>
 
+#include <math.h>
+
+#include <random>
+
 
 //-------------------------------------------------------------------------
 // synthv1widget - impl.
@@ -51,7 +57,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 {
 	Q_INIT_RESOURCE(synthv1);
 
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	// HACK: Dark themes grayed/disabled color group fix...
 	QPalette pal;
 	if (pal.base().color().value() < 0x7f) {
@@ -73,7 +79,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = NULL;
+	m_sched_notifier = nullptr;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
@@ -338,6 +344,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dco1Width2Knob, SLOT(setValue(float)));
 
 	// DCF1
+	setParamKnob(synthv1::DCF1_ENABLED,  m_ui.Dcf1GroupBox->param());
 	setParamKnob(synthv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
 	setParamKnob(synthv1::DCF1_RESO,     m_ui.Dcf1ResoKnob);
 	setParamKnob(synthv1::DCF1_TYPE,     m_ui.Dcf1TypeKnob);
@@ -398,6 +405,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dcf1Env, SLOT(setRelease(float)));
 
 	// LFO1
+	setParamKnob(synthv1::LFO1_ENABLED,	m_ui.Lfo1GroupBox->param());
 	setParamKnob(synthv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(synthv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
 	setParamKnob(synthv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
@@ -552,6 +560,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dco2Width2Knob, SLOT(setValue(float)));
 
 	// DCF2
+	setParamKnob(synthv1::DCF2_ENABLED,  m_ui.Dcf2GroupBox->param());
 	setParamKnob(synthv1::DCF2_CUTOFF,   m_ui.Dcf2CutoffKnob);
 	setParamKnob(synthv1::DCF2_RESO,     m_ui.Dcf2ResoKnob);
 	setParamKnob(synthv1::DCF2_TYPE,     m_ui.Dcf2TypeKnob);
@@ -612,6 +621,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dcf2Env, SLOT(setRelease(float)));
 
 	// LFO2
+	setParamKnob(synthv1::LFO2_ENABLED, m_ui.Lfo2GroupBox->param());
 	setParamKnob(synthv1::LFO2_SHAPE,   m_ui.Lfo2ShapeKnob);
 	setParamKnob(synthv1::LFO2_WIDTH,   m_ui.Lfo2WidthKnob);
 	setParamKnob(synthv1::LFO2_BPM,     m_ui.Lfo2BpmKnob);
@@ -755,6 +765,8 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(synthv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(synthv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
 
+	// Make status-bar keyboard range active.
+	m_ui.StatusBar->keybd()->setNoteRange(true);
 
 	// Preset management
 	QObject::connect(m_ui.Preset,
@@ -771,6 +783,11 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SLOT(resetParams()));
 
 
+	// Randomize params...
+	QObject::connect(m_ui.RandomParamsButton,
+		SIGNAL(clicked()),
+		SLOT(randomParams()));
+
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
 		SIGNAL(toggled(bool)),
@@ -783,6 +800,14 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	// Direct status-bar keyboard input
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteOnClicked(int, int)),
+		SLOT(directNoteOn(int, int)));
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteRangeChanged()),
+		SLOT(noteRangeChanged()));
+
 	// Menu actions
 	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
@@ -829,7 +854,7 @@ void synthv1widget::openSchedNotifier (void)
 		return;
 
 	synthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	m_sched_notifier = new synthv1widget_sched(pSynthUi->instance(), this);
@@ -846,7 +871,7 @@ void synthv1widget::closeSchedNotifier (void)
 {
 	if (m_sched_notifier) {
 		delete m_sched_notifier;
-		m_sched_notifier = NULL;
+		m_sched_notifier = nullptr;
 	}
 
 	synthv1_ui *pSynthUi = ui_instance();
@@ -893,19 +918,18 @@ void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_para
 
 synthv1widget_param *synthv1widget::paramKnob ( synthv1::ParamIndex index ) const
 {
-	return m_paramKnobs.value(index, NULL);
+	return m_paramKnobs.value(index, nullptr);
 }
 
 
 // Param port accessors.
-void synthv1widget::setParamValue (
-	synthv1::ParamIndex index, float fValue, bool bDefault )
+void synthv1widget::setParamValue ( synthv1::ParamIndex index, float fValue )
 {
 	++m_iUpdate;
 
 	synthv1widget_param *pParam = paramKnob(index);
 	if (pParam)
-		pParam->setValue(fValue, bDefault);
+		pParam->setValue(fValue);
 
 	updateParamEx(index, fValue);
 
@@ -955,10 +979,12 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 
 	switch (index) {
 	case synthv1::DCO1_SHAPE1:
+		m_ui.Dco1Wave1->setWaveShape(fValue);
 		m_ui.Dco1Bandl1Knob->setEnabled(
 			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
 		break;
 	case synthv1::DCO1_SHAPE2:
+		m_ui.Dco1Wave2->setWaveShape(fValue);
 		m_ui.Dco1Bandl2Knob->setEnabled(
 			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
 		break;
@@ -975,10 +1001,12 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 		}
 		break;
 	case synthv1::DCO2_SHAPE1:
+		m_ui.Dco2Wave1->setWaveShape(fValue);
 		m_ui.Dco2Bandl1Knob->setEnabled(
 			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
 		break;
 	case synthv1::DCO2_SHAPE2:
+		m_ui.Dco2Wave2->setWaveShape(fValue);
 		m_ui.Dco2Bandl2Knob->setEnabled(
 			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
 		break;
@@ -994,12 +1022,59 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 			updateParam(synthv1::DCO1_SYNC1, 0.0f);
 		}
 		break;
+	case synthv1::DCF1_ENABLED:
+		if (m_ui.Lfo1GroupBox->isChecked()) {
+			const bool bDcf1Enabled = (fValue > 0.5f);
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
+	case synthv1::LFO1_ENABLED:
+		if (fValue > 0.5f) {
+			const bool bDcf1Enabled = m_ui.Dcf1GroupBox->isChecked();
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
+	case synthv1::DCF2_ENABLED:
+		if (m_ui.Lfo2GroupBox->isChecked()) {
+			const bool bDcf2Enabled = (fValue > 0.5f);
+			m_ui.Lfo2CutoffKnob->setEnabled(bDcf2Enabled);
+			m_ui.Lfo2ResoKnob->setEnabled(bDcf2Enabled);
+		}
+		break;
+	case synthv1::LFO2_ENABLED:
+		if (fValue > 0.5f) {
+			const bool bDcf2Enabled = m_ui.Dcf2GroupBox->isChecked();
+			m_ui.Lfo2CutoffKnob->setEnabled(bDcf2Enabled);
+			m_ui.Lfo2ResoKnob->setEnabled(bDcf2Enabled);
+		}
+		break;
 	case synthv1::DCF1_SLOPE:
-		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		if (m_ui.Dcf1GroupBox->isChecked())
+			m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
 		break;
 	case synthv1::DCF2_SLOPE:
-		m_ui.Dcf2TypeKnob->setEnabled(int(fValue) != 3); // !Formant
-		// Fall thru...
+		if (m_ui.Dcf2GroupBox->isChecked())
+			m_ui.Dcf2TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		break;
+	case synthv1::LFO1_SHAPE:
+		m_ui.Lfo1Wave->setWaveShape(fValue);
+		break;
+	case synthv1::LFO2_SHAPE:
+		m_ui.Lfo2Wave->setWaveShape(fValue);
+		break;
+	case synthv1::KEY1_LOW:
+		m_ui.StatusBar->keybd()->setNoteLow(int(fValue));
+		break;
+	case synthv1::KEY1_HIGH:
+		m_ui.StatusBar->keybd()->setNoteHigh(int(fValue));
+		break;
+	case synthv1::DEF1_VELOCITY: {
+		const int vel = int(79.375f * fValue + 47.625f) & 0x7f;
+		m_ui.StatusBar->keybd()->setVelocity(vel);
+		break;
+	}
 	default:
 		break;
 	}
@@ -1015,7 +1090,7 @@ void synthv1widget::updateSchedParam ( synthv1::ParamIndex index, float fValue )
 
 	synthv1widget_param *pParam = paramKnob(index);
 	if (pParam) {
-		pParam->setValue(fValue, false);
+		pParam->setValue(fValue);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
@@ -1032,7 +1107,7 @@ void synthv1widget::updateSchedParam ( synthv1::ParamIndex index, float fValue )
 void synthv1widget::resetParams (void)
 {
 	synthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	pSynthUi->reset();
@@ -1055,6 +1130,72 @@ void synthv1widget::resetParams (void)
 }
 
 
+// Randomize params (partial).
+void synthv1widget::randomParams (void)
+{
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == nullptr)
+		return;
+
+	float p = 1.0f;
+
+	synthv1_config *pConfig = synthv1_config::getInstance();
+	if (pConfig)
+		p = 0.01f * pConfig->fRandomizePercent;
+
+	if (QMessageBox::warning(this,
+		tr("Warning"),
+		tr("About to randomize current parameter values:\n\n"
+		"-/+ %1%.\n\n"
+		"Are you sure?").arg(100.0f * p),
+		QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
+		return;
+
+	std::default_random_engine re(::time(nullptr));
+
+	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		// Filter out some non-randomizable parameters!...
+		if (index == synthv1::DCO1_OCTAVE  ||
+			index == synthv1::DCO1_TUNING  ||
+			index == synthv1::DCO1_ENVTIME ||
+			index == synthv1::DCF1_ENABLED ||
+			index == synthv1::LFO1_ENABLED) 
+			continue;
+		if (index >= synthv1::OUT1_WIDTH && index < synthv1::DCO2_SHAPE1)
+			continue;
+		if (index == synthv1::DCO2_OCTAVE  ||
+			index == synthv1::DCO2_TUNING  ||
+			index == synthv1::DCO2_ENVTIME ||
+			index == synthv1::DCF2_ENABLED ||
+			index == synthv1::LFO2_ENABLED) 
+			continue;
+		if (index >= synthv1::OUT2_WIDTH)
+			break;
+		synthv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			std::normal_distribution<float> nd;
+			const float q = p * (pParam->maximum() - pParam->minimum());
+			float fValue = pParam->value();
+			if (synthv1_param::paramFloat(index))
+				fValue += 0.5f * q * nd(re);
+			else
+				fValue = std::round(fValue + q * nd(re));
+			if (fValue < pParam->minimum())
+				fValue = pParam->minimum();
+			else
+			if (fValue > pParam->maximum())
+				fValue = pParam->maximum();
+			setParamValue(index, fValue);
+			updateParam(index, fValue);
+		}
+	}
+
+	m_ui.StatusBar->showMessage(tr("Randomize"), 5000);
+	updateDirtyPreset(true);
+}
+
+
 // Swap params A/B.
 void synthv1widget::swapParams ( bool bOn )
 {
@@ -1064,7 +1205,6 @@ void synthv1widget::swapParams ( bool bOn )
 #ifdef CONFIG_DEBUG
 	qDebug("synthv1widget::swapParams(%d)", int(bOn));
 #endif
-//	resetParamKnobs();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
@@ -1105,7 +1245,7 @@ void synthv1widget::updateParamValues (void)
 		const float fValue = (pSynthUi
 			? pSynthUi->paramValue(index)
 			: synthv1_param::paramDefaultValue(index));
-		setParamValue(index, fValue, true);
+		setParamValue(index, fValue);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
@@ -1121,7 +1261,7 @@ void synthv1widget::resetParamValues (void)
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		const float fValue = synthv1_param::paramDefaultValue(index);
-		setParamValue(index, fValue, true);
+		setParamValue(index, fValue);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
@@ -1215,6 +1355,7 @@ bool synthv1widget::queryClose (void)
 // Preset status updater.
 void synthv1widget::updateLoadPreset ( const QString& sPreset )
 {
+	resetParamKnobs();
 	updateParamValues();
 
 	m_ui.Preset->setPreset(sPreset);
@@ -1227,15 +1368,21 @@ void synthv1widget::updateLoadPreset ( const QString& sPreset )
 void synthv1widget::updateSchedNotify ( int stype, int sid )
 {
 	synthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 #ifdef CONFIG_DEBUG_0
-	qDebug("samplv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
+	qDebug("synthv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (synthv1_sched::Type(stype)) {
 	case synthv1_sched::MidiIn:
+		if (sid >= 0) {
+			const int key = (sid & 0x7f);
+			const int vel = (sid >> 7) & 0x7f;
+			m_ui.StatusBar->midiInNote(key, vel);
+		}
+		else
 		if (pSynthUi->midiInCount() > 0) {
 			m_ui.StatusBar->midiInLed(true);
 			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
@@ -1262,12 +1409,52 @@ void synthv1widget::updateSchedNotify ( int stype, int sid )
 		break;
 	}
 	case synthv1_sched::Wave:
+		if (sid > 0) {
+			updateParamValues();
+			resetParamKnobs();
+			updateDirtyPreset(false);
+		}
+		// fall thru...
 	default:
 		break;
 	}
 }
 
 
+// Direct note-on/off slot.
+void synthv1widget::directNoteOn ( int iNote, int iVelocity )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1widget::directNoteOn(%d, %d)", iNote, iVelocity);
+#endif
+
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi)
+		pSynthUi->directNoteOn(iNote, iVelocity); // note-on!
+}
+
+
+// Keyboard note range change.
+void synthv1widget::noteRangeChanged (void)
+{
+	const int iNoteLow  = m_ui.StatusBar->keybd()->noteLow();
+	const int iNoteHigh = m_ui.StatusBar->keybd()->noteHigh();
+
+#ifdef CONFIG_DEBUG
+	qDebug("padthv1widget::noteRangeChanged(%d, %d)", iNoteLow, iNoteHigh);
+#endif
+
+	updateParam(synthv1::KEY1_LOW,  float(iNoteLow));
+	updateParam(synthv1::KEY1_HIGH, float(iNoteHigh));
+
+	m_ui.StatusBar->showMessage(QString("KEY Low: %1 (%2) High: %3 (%4)")
+		.arg(synthv1_ui::noteName(iNoteLow)).arg(iNoteLow)
+		.arg(synthv1_ui::noteName(iNoteHigh)).arg(iNoteHigh), 5000);
+
+	updateDirtyPreset(true);
+}
+
+
 // MIDI In LED timeout.
 void synthv1widget::midiInLedTimeout (void)
 {
@@ -1279,7 +1466,7 @@ void synthv1widget::midiInLedTimeout (void)
 void synthv1widget::helpConfigure (void)
 {
 	synthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	synthv1widget_config(pSynthUi, this).exec();
@@ -1330,7 +1517,7 @@ void synthv1widget::helpAbout (void)
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About") + " " SYNTHV1_TITLE, sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -1358,15 +1545,15 @@ void synthv1widget::paramContextMenu ( const QPoint& pos )
 {
 	synthv1widget_param *pParam
 		= qobject_cast<synthv1widget_param *> (sender());
-	if (pParam == NULL)
+	if (pParam == nullptr)
 		return;
 
 	synthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	synthv1_controls *pControls = pSynthUi->controls();
-	if (pControls == NULL)
+	if (pControls == nullptr)
 		return;
 
 	if (!pControls->enabled())
