@@ -1,7 +1,7 @@
 // synthv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -58,8 +58,7 @@ public:
 	void closeSchedNotifier();
 
 	// Param port accessors.
-	void setParamValue(
-		synthv1::ParamIndex index, float fValue, bool bDefault = false);
+	void setParamValue(synthv1::ParamIndex index, float fValue);
 	float paramValue(synthv1::ParamIndex index) const;
 
 	// Param kbob (widget) mapper.
@@ -80,6 +79,9 @@ public slots:
 	void loadPreset(const QString& sFilename);
 	void savePreset(const QString& sFilename);
 
+	// Direct note-on/off slot.
+	void directNoteOn(int iNote, int iVelocity);
+
 protected slots:
 
 	// Preset renewal.
@@ -91,6 +93,9 @@ protected slots:
 	// Reset param knobs to default value.
 	void resetParams();
 
+	// Randomize params (partial).
+	void randomParams();
+
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
@@ -100,6 +105,9 @@ protected slots:
 	// MIDI In LED timeout.
 	void midiInLedTimeout();
 
+	// Keyboard note range change.
+	void noteRangeChanged();
+
 	// Param knob context menu.
 	void paramContextMenu(const QPoint& pos);
 
@@ -171,7 +179,7 @@ class synthv1widget_sched : public QObject
 public:
 
 	// ctor.
-	synthv1widget_sched(synthv1 *pSynth, QObject *pParent = NULL)
+	synthv1widget_sched(synthv1 *pSynth, QObject *pParent = nullptr)
 		: QObject(pParent), m_notifier(pSynth, this) {}
 
 signals:
@@ -182,12 +190,12 @@ signals:
 protected:
 
 	// Notififier visitor.
-	class Notifier : public synthv1_sched_notifier
+	class Notifier : public synthv1_sched::Notifier
 	{
 	public:
 
 		Notifier(synthv1 *pSynth, synthv1widget_sched *pSched)
-			: synthv1_sched_notifier(pSynth), m_pSched(pSched) {}
+			: synthv1_sched::Notifier(pSynth), m_pSched(pSched) {}
 
 		void notify(synthv1_sched::Type stype, int sid) const
 			{ m_pSched->emit_notify(stype, sid); }
