@@ -1,7 +1,7 @@
 // synthv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 
 #include "synthv1widget_jack.h"
 
+#include "synthv1widget_palette.h"
+
 #include "synthv1_jack.h"
 
 #ifdef CONFIG_NSM
@@ -43,7 +45,7 @@
 #endif
 #endif
 
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt4/plugins"
 #else
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt5/plugins"
@@ -58,27 +60,40 @@
 synthv1widget_jack::synthv1widget_jack ( synthv1_jack *pSynth )
 	: synthv1widget(), m_pSynth(pSynth)
 	#ifdef CONFIG_NSM
-		, m_pNsmClient(NULL)
+		, m_pNsmClient(nullptr)
 	#endif
 {
 	// Special style paths...
 	if (QDir(CONFIG_PLUGINSDIR).exists())
 		QApplication::addLibraryPath(CONFIG_PLUGINSDIR);
 
-	// Custom style theme...
+	// Custom color/style themes...
 	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig && !pConfig->sCustomStyleTheme.isEmpty())
-		QApplication::setStyle(QStyleFactory::create(pConfig->sCustomStyleTheme));
+	if (pConfig) {
+		if (!pConfig->sCustomColorTheme.isEmpty()) {
+			QPalette pal;
+			if (synthv1widget_palette::namedPalette(
+					pConfig, pConfig->sCustomColorTheme, pal))
+				QApplication::setPalette(pal);
+		}
+		if (!pConfig->sCustomStyleTheme.isEmpty()) {
+			QApplication::setStyle(
+				QStyleFactory::create(pConfig->sCustomStyleTheme));
+		}
+	}
 
 	// Initialize (user) interface stuff...
 	m_pSynthUi = new synthv1_ui(m_pSynth, false);
 
+	// Initialise preset stuff...
+	clearPreset();
+
+	// Initial update, always...
+	resetParamValues();
+	resetParamKnobs();
+
 	// May initialize the scheduler/work notifier.
 	openSchedNotifier();
-
-	// Initialize preset stuff...
-	// initPreset();
-	updateParamValues();
 }
 
 
@@ -101,10 +116,6 @@ synthv1_ui *synthv1widget_jack::ui_instance (void) const
 void synthv1widget_jack::setNsmClient ( synthv1_nsm *pNsmClient )
 {
 	m_pNsmClient = pNsmClient;
-
-	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig)
-		pConfig->bDontUseNativeDialogs = true;
 }
 
 synthv1_nsm *synthv1widget_jack::nsmClient (void) const
@@ -130,7 +141,7 @@ void synthv1widget_jack::updateDirtyPreset ( bool bDirtyPreset )
 
 #ifdef CONFIG_NSM
 	if (m_pNsmClient && m_pNsmClient->is_active())
-		m_pNsmClient->dirty(bDirtyPreset);
+		m_pNsmClient->dirty(true); // as far as NSM goes, we're always filthy!
 #endif
 }
 
@@ -158,7 +169,7 @@ void synthv1widget_jack::closeEvent ( QCloseEvent *pCloseEvent )
 // Optional GUI handlers.
 void synthv1widget_jack::showEvent ( QShowEvent *pShowEvent )
 {
-	QWidget::showEvent(pShowEvent);
+	synthv1widget::showEvent(pShowEvent);
 
 	if (m_pNsmClient)
 		m_pNsmClient->visible(true);
@@ -169,7 +180,7 @@ void synthv1widget_jack::hideEvent ( QHideEvent *pHideEvent )
 	if (m_pNsmClient)
 		m_pNsmClient->visible(false);
 
-	QWidget::hideEvent(pHideEvent);
+	synthv1widget::hideEvent(pHideEvent);
 }
 
 #endif	// CONFIG_NSM
