@@ -1,7 +1,7 @@
 // synthv1widget_preset.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -52,8 +52,8 @@ synthv1widget_preset::synthv1widget_preset ( QWidget *pParent )
 	m_pOpenButton->setIcon(QIcon(":/images/presetOpen.png"));
 	m_pComboBox->setEditable(true);
 	m_pComboBox->setMinimumWidth(240);
-#if QT_VERSION >= 0x040200
-	m_pComboBox->setCompleter(NULL);
+#if QT_VERSION >= QT_VERSION_CHECK(4, 2, 0)
+	m_pComboBox->setCompleter(nullptr);
 #endif
 	m_pComboBox->setInsertPolicy(QComboBox::NoInsert);
 	m_pSaveButton->setIcon(QIcon(":/images/presetSave.png"));
@@ -114,10 +114,6 @@ void synthv1widget_preset::clearPreset (void)
 {
 	++m_iInitPreset;
 
-	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig)
-		pConfig->sPreset.clear();
-
 	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->clearEditText();
 	m_pComboBox->blockSignals(bBlockSignals);
@@ -126,10 +122,6 @@ void synthv1widget_preset::clearPreset (void)
 
 void synthv1widget_preset::setPreset ( const QString& sPreset )
 {
-	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig)
-		pConfig->sPreset = sPreset;
-
 	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->setEditText(sPreset);
 	m_pComboBox->blockSignals(bBlockSignals);
@@ -148,14 +140,14 @@ bool synthv1widget_preset::queryPreset (void)
 		return true;
 
 	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return false;
 
 	if (m_iDirtyPreset > 0) {
-		const QString sPreset(pConfig->sPreset);
+		const QString& sPreset(pConfig->sPreset);
 		if (sPreset.isEmpty()) {
 			if (QMessageBox::warning(this,
-				tr("Warning") + " - " SYNTHV1_TITLE,
+				tr("Warning"),
 				tr("Some parameters have been changed.\n\n"
 				"Do you want to discard the changes?"),
 				QMessageBox::Discard |
@@ -163,7 +155,7 @@ bool synthv1widget_preset::queryPreset (void)
 				return false;
 		} else {
 			switch (QMessageBox::warning(this,
-				tr("Warning") + " - " SYNTHV1_TITLE,
+				tr("Warning"),
 				tr("Some preset parameters have been changed:\n\n"
 				"\"%1\".\n\nDo you want to save the changes?")
 				.arg(sPreset),
@@ -203,8 +195,8 @@ void synthv1widget_preset::loadPreset ( const QString& sPreset )
 	if (pConfig) {
 		emit loadPresetFile(pConfig->presetFile(sPreset));
 		++m_iInitPreset;
-	//	pConfig->sPreset = sPreset;
-	//	setPreset(sPreset);
+		pConfig->sPreset = sPreset;
+		setPreset(sPreset);
 		refreshPreset();
 	}
 
@@ -214,8 +206,13 @@ void synthv1widget_preset::loadPreset ( const QString& sPreset )
 
 void synthv1widget_preset::newPreset (void)
 {
-	if (queryPreset()) {
+	if (!queryPreset())
+		return;
+
+	synthv1_config *pConfig = synthv1_config::getInstance();
+	if (pConfig) {
 		emit newPresetFile();
+		pConfig->sPreset.clear();
 		clearPreset();
 		refreshPreset();
 	}
@@ -227,44 +224,55 @@ void synthv1widget_preset::newPreset (void)
 void synthv1widget_preset::openPreset (void)
 {
 	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
-	QString sFilename;
+	QStringList files;
 
 	const QString  sExt(SYNTHV1_TITLE);
-	const QString& sTitle  = tr("Open Preset") + " - " SYNTHV1_TITLE;
+	const QString& sTitle  = tr("Open Preset");
 	const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
-#if 1//QT_VERSION < 0x040400
+
+	QWidget *pParentWidget = nullptr;
 	QFileDialog::Options options = 0;
-	if (pConfig->bDontUseNativeDialogs)
+	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
-	sFilename = QFileDialog::getOpenFileName(parentWidget(),
-		sTitle, pConfig->sPresetDir, sFilter, NULL, options);
+		pParentWidget = QWidget::window();
+	}
+#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
+	files = QFileDialog::getOpenFileNames(pParentWidget,
+		sTitle, pConfig->sPresetDir, sFilter, nullptr, options);
 #else
-	QFileDialog fileDialog(nativeParentWidget(),
+	QFileDialog fileDialog(pParentWidget,
 		sTitle, pConfig->sPresetDir, sFilter);
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
-	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	fileDialog.setFileMode(QFileDialog::ExistingFiles);
 	fileDialog.setDefaultSuffix(sExt);
 	QList<QUrl> urls(fileDialog.sidebarUrls());
 	urls.append(QUrl::fromLocalFile(pConfig->sPresetDir));
 	fileDialog.setSidebarUrls(urls);
-	if (pConfig->bDontUseNativeDialogs)
-		fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
+	fileDialog.setOptions(options);
 	if (fileDialog.exec())
-		sFilename = fileDialog.selectedFiles().first();
+		files = fileDialog.selectedFiles();
 #endif
-	if (!sFilename.isEmpty()) {
-		QFileInfo fi(sFilename);
-		if (fi.exists() && queryPreset()) {
-			const QString& sPreset = fi.completeBaseName();
-			pConfig->setPresetFile(sPreset, sFilename);
-			emit loadPresetFile(sFilename);
-			++m_iInitPreset;
-		//	pConfig->sPreset = sPreset;
-			pConfig->sPresetDir = fi.absolutePath();
-			setPreset(sPreset);
+
+	if (!files.isEmpty() && queryPreset()) {
+		int iPreset = 0;
+		QStringListIterator iter(files);
+		while (iter.hasNext()) {
+			const QString& sFilename = iter.next();
+			const QFileInfo fi(sFilename);
+			if (fi.exists()) {
+				const QString& sPreset = fi.completeBaseName();
+				pConfig->setPresetFile(sPreset, sFilename);
+				if (++iPreset == 1) {
+					++m_iInitPreset;
+					emit loadPresetFile(sFilename);
+					pConfig->sPreset = sPreset;
+					pConfig->sPresetDir = fi.absolutePath();
+					setPreset(sPreset);
+				}
+			}
 			refreshPreset();
 		}
 	}
@@ -284,23 +292,26 @@ void synthv1widget_preset::savePreset ( const QString& sPreset )
 		return;
 
 	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	const QString sExt(SYNTHV1_TITLE);
 	QFileInfo fi(QDir(pConfig->sPresetDir), sPreset + '.' + sExt);
 	QString sFilename = fi.absoluteFilePath();
 	if (!fi.exists()) {
-		const QString& sTitle  = tr("Save Preset") + " - " SYNTHV1_TITLE;
+		const QString& sTitle  = tr("Save Preset");
 		const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
-	#if 1//QT_VERSION < 0x040400
+		QWidget *pParentWidget = nullptr;
 		QFileDialog::Options options = 0;
-		if (pConfig->bDontUseNativeDialogs)
+		if (pConfig->bDontUseNativeDialogs) {
 			options |= QFileDialog::DontUseNativeDialog;
-		sFilename = QFileDialog::getSaveFileName(parentWidget(),
-			sTitle, sFilename, sFilter, NULL, options);
+			pParentWidget = QWidget::window();
+		}
+	#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
+		sFilename = QFileDialog::getSaveFileName(pParentWidget,
+			sTitle, sFilename, sFilter, nullptr, options);
 	#else
-		QFileDialog fileDialog(nativeParentWidget(),
+		QFileDialog fileDialog(pParentWidget,
 			sTitle, sFilename, sFilter);
 		fileDialog.setAcceptMode(QFileDialog::AcceptSave);
 		fileDialog.setFileMode(QFileDialog::AnyFile);
@@ -308,14 +319,13 @@ void synthv1widget_preset::savePreset ( const QString& sPreset )
 		QList<QUrl> urls(fileDialog.sidebarUrls());
 		urls.append(QUrl::fromLocalFile(pConfig->sPresetDir));
 		fileDialog.setSidebarUrls(urls);
-		if (pConfig->bDontUseNativeDialogs)
-			fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
+		fileDialog.setOptions(options);
 		if (fileDialog.exec())
 			sFilename = fileDialog.selectedFiles().first();
 	#endif
 	} else {
-		if (QMessageBox::warning(parentWidget(),
-			tr("Warning") + " - " SYNTHV1_TITLE,
+		if (QMessageBox::warning(QWidget::window(),
+			tr("Warning"),
 			tr("About to replace preset:\n\n"
 			"\"%1\"\n\n"
 			"Are you sure?")
@@ -348,11 +358,10 @@ void synthv1widget_preset::deletePreset (void)
 		return;
 
 	synthv1_config *pConfig = synthv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
-
-	if (QMessageBox::warning(parentWidget(),
-		tr("Warning") + " - " SYNTHV1_TITLE,
+	if (QMessageBox::warning(QWidget::window(),
+		tr("Warning"),
 		tr("About to remove preset:\n\n"
 		"\"%1\"\n\n"
 		"Are you sure?")
@@ -362,6 +371,7 @@ void synthv1widget_preset::deletePreset (void)
 		return;
 
 	pConfig->removePreset(sPreset);
+	pConfig->sPreset.clear();
 
 	clearPreset();
 	refreshPreset();
